/*
 * This file is part of the DOM implementation for KDE.
 *
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 * Copyright (C) 2003 Dirk Mueller (mueller@kde.org)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */
#ifndef _DOM_DOMStringImpl_h_
#define _DOM_DOMStringImpl_h_

#include <qstring.h>

#include "dom/dom_misc.h"
#include "misc/khtmllayout.h"
#include "misc/shared.h"

namespace DOM {

class DOMStringImpl : public khtml::Shared<DOMStringImpl>
{
private:
    DOMStringImpl(const DOMStringImpl&);
    DOMStringImpl& operator=(const DOMStringImpl&);
public:
    DOMStringImpl();
    DOMStringImpl(const QChar *str, unsigned int len);
    DOMStringImpl( const ushort* str, const uint& len );
    DOMStringImpl(const char *str);
    DOMStringImpl(const QChar &ch);
    ~DOMStringImpl();

    void append(DOMStringImpl *str);
    void insert(DOMStringImpl *str, unsigned int pos);
    void truncate(int len);
    void remove(unsigned int pos, int len=1);

    DOMStringImpl *split(unsigned int pos);
    DOMStringImpl *copy() const;
    DOMStringImpl *substring(unsigned int pos, unsigned int len);
    DOMStringImpl *collapseWhiteSpace(bool preserveLF, bool preserveWS);

    const QChar operator [] (int pos);

    void setChar( const QChar& v, int pos );
    
    const QConstString constString();

    const ushort* unicode();
    bool containsOnlyWhitespace() const;

    // ignores trailing garbage, unlike QString
    int toInt(bool* ok = 0) const;

    khtml::Length* toLengthArray(int& len) const;
    khtml::Length* toCoordsArray(int& len) const;
    bool isLower() const;
    DOMStringImpl *lower() const;
    DOMStringImpl *upper() const;
    DOMStringImpl *capitalize() const;

    unsigned int l;  // This shouldn't be public!! (se)
private:
    QChar *s;
    ushort* ushort_s;
};

}
#endif
